import '../log/log.dart';
import 'data_classes/active_case.dart';
import 'repositories/active_cases_repository_local.dart';
import 'repositories/active_cases_repository_remote.dart';

class ActiveCases
{
  static ActiveCases _instance = ActiveCases._internal();
  factory ActiveCases() => _instance;
  ActiveCases._internal()
  {
    _repositoryLocal = ActiveCasesRepositoryLocal();
    _repositoryRemote = ActiveCasesRepositoryRemote();
  }

  bool _isLoaded = false;
  bool get isLoaded => _isLoaded;

  ActiveCasesRepositoryLocal _repositoryLocal;
  ActiveCasesRepositoryRemote _repositoryRemote;

  List<ActiveCase> get activeCases => _activeCases;
  List<ActiveCase> _activeCases = List<ActiveCase>();

  List<ActiveCase> get remoteActiveCases => _remoteActiveCases;
  List<ActiveCase> _remoteActiveCases = List<ActiveCase>();

  DateTime get lastTimeLoadedRemoteActiveCases => _lastTimeLoadedRemoteActiveCases;
  DateTime _lastTimeLoadedRemoteActiveCases = DateTime(2000);

  Duration get loadRemoteActiveCasesFrequency => _loadRemoteActiveCasesFrequency;
  set loadRemoteActiveCasesFrequency(Duration duration) => _loadRemoteActiveCasesFrequency = duration;
  Duration _loadRemoteActiveCasesFrequency = Duration(days: 1);

  String _cachedUserID = "";

  Future<bool> load({String userID = "", Duration removeOlderThan = null}) async
  {
    await _loadActiveCasesFromLocal();

    if (removeOlderThan != null)
      removeActiveCasesOlderThan(removeOlderThan);

    if (_cachedUserID == "" && userID == "")
      return false;
    else if (userID != "")
      _cachedUserID = userID;

    await _loadActiveCasesFromRemote(_cachedUserID);

    _isLoaded = true;

    return _isLoaded;
  }

  Future<bool> loadRemote({String userID = ""}) async
  {
    if (_cachedUserID == "" && userID == "")
      return false;
    else if (userID != "")
      _cachedUserID = userID;

    return await _loadActiveCasesFromRemote(_cachedUserID);
  }

  Future<bool> _loadActiveCasesFromLocal() async
  {
    final result = await _repositoryLocal.getActiveCases();
    if (result != null) {
      _activeCases = result;
      print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Loaded " + result.length.toString() + " active cases from local database into active cases list.");
      return true;
    }
    else {
      print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Could not load active cases from local database into active cases list.");
    }
    return false;
  }

  Future<bool> saveActiveCasesToLocal() async
  {
    final result = await _repositoryLocal.saveActiveCases(_activeCases);
    if (result)
      print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Saved " + _activeCases.length.toString() + " active cases from list to local database.");
    else
      print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Could not save active cases from list to local database.");
    return result;
  }

  Future<bool> _loadActiveCasesFromRemote(String userID) async
  {
    final result = await _repositoryRemote.getActiveCases(userID: userID);

    if (result != null) {
      _remoteActiveCases = result;
      _lastTimeLoadedRemoteActiveCases = DateTime.now();
      print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Loaded " + result.length.toString() + " active cases from remote database into remote active cases list.");
      return true;
    }
    else {
      print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Could not load active cases from remote database into remote active cases list.");
    }
    return false;
  }

  void addActiveCases(List<ActiveCase> activeCasesToAdd)
  {
    for (ActiveCase activeCase in activeCasesToAdd) {
      final match = _activeCases.firstWhere((ac) => ac.id == activeCase.id, orElse: () => null);
      if (match != null)
        _activeCases.add(activeCase);
    }
    Log.addToLog("Added new active cases to local list, now containing " + _activeCases.length.toString() + " cases.");
    saveActiveCasesToLocal();
  }

  void removeActiveCasesOlderThan(Duration duration)
  {
    for (int i=_activeCases.length-1; i>=0; i--) {
      if (DateTime.now().difference(_activeCases[i].startDate) >= duration)
        _activeCases.removeAt(i);
    }
    print("HEROES_FLUTTER_PLUGIN :: ActiveCases :: Removed old (over " + duration.inDays.toString() + " days) active cases from local list, now containing " + _activeCases.length.toString() + " cases.");
    Log.addToLog("Removed old (over " + duration.inDays.toString() + " days) active cases from local list, now containing " + _activeCases.length.toString() + " cases.");
    saveActiveCasesToLocal();
  }
}