import '../../configuration/configuration_service.dart';
import '../../configuration/data_classes/configuration.dart';
import '../../log/log.dart';
import '../data_classes/active_case.dart';
import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:meta/meta.dart';

class ActiveCasesDataSourceFirebase
{
  String _baseURL = "";
  String _suffixForGetActiveCases = "";

  ActiveCasesDataSourceFirebase()
  {
    ConfigurationService configurationService = ConfigurationService();
    if (configurationService.isLoaded) {
      _baseURL = configurationService.configuration.baseURL;
      _suffixForGetActiveCases = configurationService.configuration.suffixForGettingAllActiveCases;
    }
    configurationService.addOnConfigurationChangedCallback(_onConfigurationChangedCallback);
  }

  /// IF THERE WAS A PROBLEM CONNECTING TO SERVER, WILL RETURN NULL.
  Future<List<ActiveCase>> getActiveCases({@required String userID}) async
  {
    List<ActiveCase> remoteActiveCases = new List<ActiveCase>();

    try
    {
      final response = await http.get(
        _baseURL + _suffixForGetActiveCases,
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "userID": userID,
        },
      );
      if (response.statusCode == 200) {
        List<dynamic> result = json.decode(response.body);
        for(int i=0; i < result.length; i++) {
          final id = result[i]["id"];
          final startDateInMillisecondsSinceJan1st1970 = int.parse(result[i]["startDate"]);
          final startDate = DateTime(1970).add(Duration(milliseconds: startDateInMillisecondsSinceJan1st1970));
          remoteActiveCases.add(ActiveCase(id: id, startDate: startDate));
        }
        Log.addToLog("Downloaded " + result.length.toString() + " active cases from the server.");

        return remoteActiveCases;
      }
      else {
        print("HEROES_FLUTTER_PLUGIN :: ActiveCasesDataSourceFirebase :: Couldn't get remote active cases list. Server response = " + response.statusCode.toString());
        Log.addToLog("Could not download active cases list. Server response = " + response.statusCode.toString() + ".");
      }
    }
    catch (error)
    {
      print("HEROES_FLUTTER_PLUGIN :: ActiveCasesDataSourceFirebase :: Error getting or parsing remote active cases list.");
      Log.addToLog("Error downloading or parsing active cases list from the server.");
    }
    return null;
  }

  void _onConfigurationChangedCallback(Configuration newConfiguration)
  {
    final newBaseURL = newConfiguration.baseURL;
    if (_baseURL != newBaseURL)
    {
      Log.addToLog("baseURL changed: \"" + _baseURL + "\" -> \"" + newBaseURL + "\"");
      _baseURL = newBaseURL;
    }

    final newSuffixForGetActiveCases = newConfiguration.suffixForGettingAllActiveCases;
    if (_suffixForGetActiveCases != newSuffixForGetActiveCases)
    {
      Log.addToLog("suffixForGetActiveCases changed: \"" + _suffixForGetActiveCases + "\" -> \"" + newSuffixForGetActiveCases + "\"");
      _suffixForGetActiveCases = newSuffixForGetActiveCases;
    }
  }
}