import '../../log/log.dart';
import '../data_classes/active_case.dart';
import 'dart:convert';
import 'dart:io';
import 'package:path_provider/path_provider.dart';

class ActiveCasesDataSourceLocal
{
  /// THROWS EXCEPTION IF ANYTHING BAD HAPPENS.
  Future<List<ActiveCase>> getActiveCases() async
  {
    final file = await _localFile;
    if (await file.exists()) {
      final activeCasesAsJsonString = file.readAsStringSync();

      try {
        List<dynamic> activeCasesAsListOfDynamic = json.decode(activeCasesAsJsonString);
        if (activeCasesAsListOfDynamic != null) {
          final activeCasesAsList = List<ActiveCase>();
          for (int i = 0; i < activeCasesAsListOfDynamic.length; i++)
            activeCasesAsList.add(
                ActiveCase.fromJson(activeCasesAsListOfDynamic[i]));
          return activeCasesAsList;
        }
        print("HEROES_FLUTTER_PLUGIN :: ActiveCasesDataSourceLocal :: Loaded " + activeCasesAsListOfDynamic.length.toString() + " active cases from local database.");
        Log.addToLog("Loaded " + activeCasesAsListOfDynamic.length.toString() + " active cases from local database.");
        return List<ActiveCase>();
      }
      catch (error) {
        print("HEROES_FLUTTER_PLUGIN :: ActiveCasesDataSourceLocal :: Local active cases database is empty, or could not parse it.");
        Log.addToLog("Local active cases database is empty, or could not parse it.");
        return List<ActiveCase>();
      }
    }
    else {
      await file.create();
      final path = await _localPath;
      print("File $path/$_fileName did not exist, created it.");
      Log.addToLog("Local active cases database file did not exist.");
      return List<ActiveCase>();
    }
  }

  Future<bool> saveActiveCases(List<ActiveCase> activeCases) async
  {
    try {
      File file = await _localFile;

      if (!(await file.exists()))
        file = await file.create();

      List<dynamic> activeCasesAsJson = List<dynamic>();
      for (int i = 0; i < activeCases.length; i++)
        activeCasesAsJson.add(activeCases[i].toJson());
      final activeCasesAsJsonString = json.encode(activeCasesAsJson);

      file.writeAsStringSync(activeCasesAsJsonString);
      Log.addToLog("Saved " + activeCases.length.toString() + " active cases to local database.");

      return true;
    }
    catch (error) {
      try {
        final path = await _localPath;
        File file = new File("$path/$_fileName");

        if (!(await file.exists()))
          file = await file.create();

        List<dynamic> activeCasesAsJson = List<dynamic>();
        for (int i = 0; i < activeCases.length; i++)
          activeCasesAsJson.add(activeCases[i].toJson());
        final activeCasesAsJsonString = json.encode(activeCasesAsJson);

        file.writeAsStringSync(activeCasesAsJsonString);
        Log.addToLog("Saved " + activeCases.length.toString() + " active cases to local database.");

        return true;
      }
      catch (error) {
        final path = await _localPath;
        print("Could not save " + activeCases.length.toString() + " active cases to file $path/$_fileName.");
        Log.addToLog("Could not save " + activeCases.length.toString() + " active cases to local database.");
        return false;
      }
    }
  }

  final String _fileName = "Heroes_active_cases.json";

  Future<String> get _localPath async {
    final directory = await getApplicationDocumentsDirectory();
    return directory.path;
  }

  Future<File> get _localFile async {
    final path = await _localPath;
    return File('$path/$_fileName');
  }
}