import '../log/log.dart';
import 'data_classes/configuration.dart';
import 'data_sources/configuration_data_source_firebase.dart';
import 'package:meta/meta.dart';

class ConfigurationService
{
  static final _instance = ConfigurationService._internal();
  factory ConfigurationService() => _instance;
  ConfigurationService._internal();

  ConfigurationDataSourceFirebase _dataSource = ConfigurationDataSourceFirebase();

  bool _isLoaded = false;
  bool get isLoaded => _isLoaded;

  Configuration _configuration;
  Configuration get configuration => _configuration;

  Future<bool> loadConfiguration({@required String APIkey}) async
  {
    final result = await _dataSource.getConfiguration(APIkey: APIkey);

    if (result != null) {
      _configuration = result;
      _isLoaded = true;
      for (Function callback in _onConfigurationChangedCallbacks)
        callback(configuration);
    }

    Log.addToLog("Configuration is loaded.");
    return _isLoaded;
  }

  final _onConfigurationChangedCallbacks = List<Function>();

  void addOnConfigurationChangedCallback(Function function)
  {
    if (!_onConfigurationChangedCallbacks.contains(function))
      _onConfigurationChangedCallbacks.add(function);
  }

  void removeOnConfigurationChangedCallback(Function function)
  {
    if (_onConfigurationChangedCallbacks.contains(function))
      _onConfigurationChangedCallbacks.remove(function);
  }
}