import '../../log/log.dart';
import '../data_classes/configuration.dart';
import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:meta/meta.dart';

class ConfigurationDataSourceFirebase
{
  final String url = "https://heroesapp-2067e.web.app/api/v1/";

  /// IF THERE WAS A PROBLEM CONNECTING TO SERVER, WILL RETURN NULL.
  Future<Configuration> getConfiguration({@required String APIkey}) async
  {
    try
    {
      final response = await http.get(
        url + "getConfiguration/",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "APIkey": APIkey,
        },
      );
      if (response.statusCode == 200) {
        Map<String, dynamic> result = json.decode(response.body);
        final configuration = Configuration(
          baseURL : result["baseURL"],
          suffixForRegisteringUser : result["suffixForRegisteringUser"],
          suffixForGettingAllActiveCases : result["suffixForGettingAllActiveCases"],
          serviceID : result["serviceID"],
          encounterIsRelevantWindow: Duration(seconds: result["encounterIsRelevantWindow"]),
          rebootFrequency: Duration(seconds: result["rebootFrequency"]),
          encounterWindow: Duration(seconds: result["encounterWindow"]),
          backUpFrequency: Duration(seconds: result["backUpFrequency"]),
          loadRemoteActiveCasesFrequency: Duration(seconds: result["loadRemoteActiveCasesFrequency"]),
          maxTimeWithoutEventsForReboot: Duration(seconds: result["maxTimeWithoutEventsForReboot"]),
        );
        Log.addToLog("Downloaded configuration from server.");
        return configuration;
      }
      else {
        print("HEROES_FLUTTER_PLUGIN :: ConfigurationDataSourceFirebase :: Could not get configuration from server. Server response = " + response.statusCode.toString());
        Log.addToLog("Could not get configuration from server. Server response = " + response.statusCode.toString() + ".");
      }
    }
    catch (error)
    {
      print("HEROES_FLUTTER_PLUGIN :: ConfigurationDataSourceFirebase :: Error getting or parsing configuration from server.");
      Log.addToLog("Error getting or parsing configuration from server.");
    }
    return null;
  }
}