import '../../log/log.dart';
import '../data_classes/encounter_entry.dart';
import '../data_classes/ended_encounters_json_transformer.dart';
import 'dart:convert';
import 'dart:io';
import 'package:path_provider/path_provider.dart';

class EncountersDataSource
{
  /// THROWS EXCEPTION IF ANYTHING BAD HAPPENS.
  Future<Map<String, List<EncounterEntry>>> getEncounters() async
  {
    final file = await _localFile;
    if (await file.exists())
    {
      final encountersAsJsonString = file.readAsStringSync();
      try {
        Map<String, dynamic> encountersAsJson = json.decode(encountersAsJsonString);
        final encountersMap = EndedEncountersJsonTransformer.fromJson(encountersAsJson);
        if (encountersMap != null)
          return encountersMap;
        print("HEROES_FLUTTER_PLUGIN :: Local encounters database is empty, or could not parse it.");
        return Map<String, List<EncounterEntry>>();
      }
      catch (error) {
        print("HEROES_FLUTTER_PLUGIN :: Local encounters database is empty, or could not parse it.");
        return Map<String, List<EncounterEntry>>();
      }
    }
    else
    {
      await file.create();
      final path = await _localPath;
      print("HEROES_FLUTTER_PLUGIN :: File $path/$_fileName did not exist, created it.");
      return Map<String, List<EncounterEntry>>();
    }
  }

  Future<bool> saveEncounters(Map<String, List<EncounterEntry>> encountersMap) async
  {
    try {
      File file = await _localFile;

      if (!(await file.exists()))
        file = await file.create();

      Map<String, dynamic> encountersAsJson = EndedEncountersJsonTransformer.toJson(encountersMap);
      final encountersAsJsonString = json.encode(encountersAsJson);

      file.writeAsStringSync(encountersAsJsonString);
      Log.addToLog("Saved " + encountersMap.entries.length.toString() + " enocounters to local database.");

      return true;
    }
    catch (error) {
      try {
        final path = await _localPath;
        File file = new File("$path/$_fileName");

        if (!(await file.exists()))
          file = await file.create();

        Map<String, dynamic> encountersAsJson = EndedEncountersJsonTransformer.toJson(encountersMap);
        final encountersAsJsonString = json.encode(encountersAsJson);

        file.writeAsStringSync(encountersAsJsonString);
        Log.addToLog("Saved " + encountersMap.entries.length.toString() + " encounters to local database.");

        return true;
      }
      catch (error) {
        final path = await _localPath;
        print("HEROES_FLUTTER_PLUGIN :: Could not save encounters map to file $path/$_fileName.");
        return false;
      }
    }
  }

  final String _fileName = "cvapp_data.json";
  final String _directory = "-";

  Future<String> get _localPath async {
    final directory = await getApplicationDocumentsDirectory();
    return directory.path;
  }

  Future<File> get _localFile async {
    final path = _directory != "-" ? _directory : await _localPath;
    return File('$path/$_fileName');
  }
}