import 'package:geolocator/geolocator.dart';
import 'dart:async';
import '../data_service/data_service.dart';
import '../utils_service/utils_service.dart';
import 'package:heroes/src/heroes_class.dart';

class GPSService {
  factory GPSService() => _instance;

  GPSService._internal();

  static final _instance = GPSService._internal();

  Position deviceLocation;
  Timer initSubscription;

  double latitude;
  double longitude;

  Future<void> init() async {
    Geolocator geolocator = Geolocator()..forceAndroidLocationManager = true;
    bool permissionGranted = await UtilsService().checkLocationPermissions();

    if (permissionGranted) {

      if(initSubscription != null)
        initSubscription.cancel();
        
      await UtilsService().checkLocationServiceEnabled();
      var locationOptions =
          LocationOptions(accuracy: LocationAccuracy.high, distanceFilter: 10);
      StreamSubscription<Position> positionStream = geolocator
          .getPositionStream(locationOptions)
          .listen((Position position) {
        if (position != null) {
          latitude = position.latitude;
          longitude = position.longitude;
          deviceLocation = position;
        }

        print(position == null
            ? 'HEROES_FLUTTER_PLUGIN :: APP_LOG :: Device position unknown'
            : "HEROES_FLUTTER_PLUGIN :: APP_LOG :: Device position: " +
                position.latitude.toString() +
                ', ' +
                position.longitude.toString());
      });
    } else {

      if(initSubscription == null || !initSubscription.isActive)
        initSubscription =  Timer.periodic(Duration(seconds: 1), (Timer t) => init);
    }
    return null;
  }

  Future<void> getCurrentLocation() async {
    deviceLocation = await Geolocator()
        .getLastKnownPosition(desiredAccuracy: LocationAccuracy.high);
    return null;
  }
}
