import '../data_service/data_service.dart';
import '../encounters/data_classes/encounter_entry.dart';
import '../log/log.dart';
import 'dart:async';
import 'package:flutter/material.dart';

class HeroesDebugWidget extends StatefulWidget
{
  @override
  _DebugState createState() => _DebugState();
}

class _DebugState extends State<HeroesDebugWidget>
{
  final _dataService = DataService();

  List<EncounterEntry> _currentEncounters = List<EncounterEntry>();
  List<Widget> _logMessages = List<Widget>();

  Timer _debugTimer;

  void _updateLogMessages(Timer t)
  {
    final messages = List<Widget>();
    final logs = Log.log;
    for (int i = 0; i < logs.length; i++) messages.add(Text(logs[i]));
    setState(() {
      _logMessages = messages;
      _currentEncounters = _dataService.getCurrentEncounters();
    });
  }

  @override
  void initState()
  {
    super.initState();
    _updateLogMessages(null);
    _debugTimer = Timer.periodic(Duration(seconds: 1), _updateLogMessages);
  }

  @override
  void dispose()
  {
    if (_debugTimer != null)
      _debugTimer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context)
  {
    return Column(
      children: <Widget>[
        Container(
          padding: EdgeInsets.only(top: 10),
          child: Text('Current encounters: ' + _currentEncounters.length.toString())
        ),
        Container(
          padding: EdgeInsets.fromLTRB(10, 0, 10, 0),
          child: ListView.builder(
            primary: false,
            shrinkWrap: true,
            reverse: false,
            itemCount: _currentEncounters.length,
            itemBuilder: (context, index) {
              return Container(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: <Widget>[
                    Center(
                      child: Text(
                        "ID: " + _currentEncounters[index].theirId,
                        style: TextStyle(
                          color: _dataService.isSick(_currentEncounters[index].theirId) ? Colors.red : Colors.green)
                        ),
                      ),
                      Container(height: 10)
                  ]
                ),
              );
            }
          ),
        ),
        Container(
          height: 20,
          child: Text("System Log")
        ),
        Container(
          height: 280,
          decoration: myBoxDecoration(),
          padding: EdgeInsets.fromLTRB(10, 0, 10, 0),
          child: ListView(
            primary: false,
            children: _logMessages,
          ),
        ),
      ],
    );
  }

  BoxDecoration myBoxDecoration() {
    return BoxDecoration(
      border: Border.all(
        width: 1,
      ),
    );
  }
}