import '../../configuration/configuration_service.dart';
import '../../configuration/data_classes/configuration.dart';
import '../../log/log.dart';
import '../data_classes/registration_result.dart';
import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:meta/meta.dart';

class RegistrationDataSourceFirebase
{
  String _baseURL = "";
  String _suffixForRegisterUser = "";

  RegistrationDataSourceFirebase()
  {
    ConfigurationService configurationService = ConfigurationService();
    if (configurationService.isLoaded) {
      _baseURL = configurationService.configuration.baseURL;
      _suffixForRegisterUser = configurationService.configuration.suffixForRegisteringUser;
    }
    configurationService.addOnConfigurationChangedCallback(_onConfigurationChangedCallback);
  }

  Future<RegistrationResult> registerUser({@required inviteCode}) async
  {
    try
    {
      final response = await http.get(
        _baseURL + _suffixForRegisterUser,
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "invitationCode": inviteCode,
        },
      );
      if (response.statusCode == 200) {
        String id = json.decode(response.body)["id"].toString();
        final invitationCodes = List<String>();
        invitationCodes.add(json.decode(response.body)["userInvitationCode1"].toString());
        invitationCodes.add(json.decode(response.body)["userInvitationCode2"].toString());
        invitationCodes.add(json.decode(response.body)["userInvitationCode3"].toString());

        Log.addToLog("Successfully registered user. ID = " + id.substring(0, 6) + ".");
        return RegistrationResult(true, id, invitationCodes);
      }
      else {
        print("RegistrationDataSourceFirebase :: Couldn't register myself with invite code " + inviteCode + ". Server response = " + response.statusCode.toString());
        Log.addToLog("Couldn't register user with invite code " + inviteCode + ". Server response = " + response.statusCode.toString() + ".");
      }
    }
    catch (error)
    {
      print("RegistrationDataSourceFirebase :: Error registering myself with invite code " + inviteCode + ": " + error.toString());
      Log.addToLog("Error registering user with invite code " + inviteCode + ".");
    }

    return RegistrationResult(false, null, null);
  }

  void _onConfigurationChangedCallback(Configuration newConfiguration)
  {
    final newBaseURL = newConfiguration.baseURL;
    if (_baseURL != newBaseURL)
    {
      Log.addToLog("baseURL changed: \"" + _baseURL + "\" -> \"" + newBaseURL + "\"");
      _baseURL = newBaseURL;
    }

    final newSuffixForRegisterUser = newConfiguration.suffixForRegisteringUser;
    if (_suffixForRegisterUser != newSuffixForRegisterUser)
    {
      Log.addToLog("suffixForRegisterUser changed: \"" + _suffixForRegisterUser + "\" -> \"" + newSuffixForRegisterUser + "\"");
      _suffixForRegisterUser = newSuffixForRegisterUser;
    }
  }
}