import 'data_classes/registration_result.dart';
import 'data_sources/registration_data_source_firebase.dart';
import 'data_sources/registration_data_source_shared_preferences.dart';
import 'package:meta/meta.dart';

class Registration
{
  static Registration _instance = Registration._internal();
  factory Registration() => _instance;
  Registration._internal() {
    _registrationFirebaseDataSource = RegistrationDataSourceFirebase();
    _userID = "placeholder ID";
  }

  String _userID;
  String get userID => _userID;

  bool _hasDummyID = true;

  RegistrationDataSourceFirebase _registrationFirebaseDataSource;
  RegistrationDataSourceSharedPreferences _registrationSharedPreferencesDataSource;

  Future<bool> get isRegistered async
  {
    if (_registrationSharedPreferencesDataSource == null)
      _registrationSharedPreferencesDataSource = await RegistrationDataSourceSharedPreferences.getNewInstance();

    final invitationCodes = _registrationSharedPreferencesDataSource.getInvitationCodes();
    if (invitationCodes != null) {
      _invitationCodes = invitationCodes;
    }

    final userID = _registrationSharedPreferencesDataSource.getUserID();
    if (userID != null) {
      _userID = userID;
      _hasDummyID = false;
      return true;
    }
    else if (_userID != null && !_hasDummyID) {
      _registrationSharedPreferencesDataSource.setUserID(_userID);
      return true;
    }

    return false;
  }

  List<String> _invitationCodes = List<String>();
  List<String> get invitationCodes => _invitationCodes;

  Future<RegistrationResult> registerUser({@required String inviteCode}) async
  {
    final result = await _registrationFirebaseDataSource.registerUser(inviteCode: inviteCode);

    if (result == null)
      return RegistrationResult(false, null, null);

    if (result.success)
    {
      _userID = result.userID;
      _invitationCodes = result.invitationCodes;
      _hasDummyID = false;

      if (_registrationSharedPreferencesDataSource == null)
        _registrationSharedPreferencesDataSource = await RegistrationDataSourceSharedPreferences.getNewInstance();

      _registrationSharedPreferencesDataSource.setUserID(result.userID);
      _registrationSharedPreferencesDataSource.setInvitationCodes(result.invitationCodes);
    }

    return result;
  }
}